extends CharacterBody2D


@export var SPEED = 300.0
@export var JUMP_VELOCITY = -300.0
@export var BUMP_FORCE = 80.0
@export var PUSH_FORCE = 750

@onready var right_ray = $RightRay
@onready var left_ray = $LeftRay
@onready var marker_right = $MarkerRight
@onready var marker_left = $MarkerLeft

var faceLeft = false
var pushLeftEnabled = false
var pushRightEnabled = false
var pushTarget

# Get the gravity from the project settings to be synced with RigidBody nodes.
var gravity = ProjectSettings.get_setting("physics/2d/default_gravity")

func _physics_process(delta):
		# Add the gravity.
	if not is_on_floor():
		velocity.y += gravity * delta

	# Handle jump.
	if Input.is_action_just_pressed("ui_accept") and is_on_floor():
		velocity.y = JUMP_VELOCITY
	# Hand force push (melee attack)
	if Input.is_action_just_pressed("push") && pushRightEnabled && faceLeft == false:
		pushTarget.apply_central_impulse(Vector2(1,0) * PUSH_FORCE * 10)
		pushRightEnabled = false
	if Input.is_action_just_pressed("push") && pushLeftEnabled:
		pushTarget.apply_central_impulse(Vector2(-1,0) * PUSH_FORCE * 10)
		pushLeftEnabled = false
		
	if Input.is_action_just_pressed("shoot"):
		#make a bullet
		var myBullet = %SceneManager.bulletFactory()
		#set bullet speed?
		myBullet.transform = marker_right.global_transform
	# Get the input direction and handle the movement/deceleration.
	# As good practice, you should replace UI actions with custom gameplay actions.
	var direction = Input.get_axis("ui_left", "ui_right")
	if direction <0:
		faceLeft = true
	if direction >0:
		faceLeft = false
		
	if direction:
		velocity.x = direction * SPEED
	else:
		velocity.x = move_toward(velocity.x, 0, SPEED)

	move_and_slide()
	if right_ray.is_colliding():
		print("right ray is colliding")
		var collider = right_ray.get_collider()
		#what did we hit?
		if collider is Node:
			if collider.is_in_group("boxes"):
				print("This is a pushable box")
				pushRightEnabled = true
				pushTarget = collider
	else:
		pushRightEnabled = false
		
	if left_ray.is_colliding():
		print("left ray is colliding")
		var collider = left_ray.get_collider()
		if collider is Node:
			if collider.is_in_group("boxes"):
				pushLeftEnabled = true
				pushTarget = collider
	else:
		pushLeftEnabled = false
				
	#loop through the collisions
	for i in get_slide_collision_count():
		var c = get_slide_collision(i)
		if c.get_collider() is RigidBody2D:
			c.get_collider().apply_central_impulse(-c.get_normal()* BUMP_FORCE)
